# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: install.py 314 2008-06-26 22:02:34Z flarumbe $
#
# end: Platecom header
"""
$Id: install.py 314 2008-06-26 22:02:34Z flarumbe $
"""
from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import shasattr
from Products.Archetypes.Extensions.utils import install_subskin
from icsemantic.thesaurus.config import *
from icsemantic.thesaurus.interfaces import IThesaurus
from icsemantic.thesaurus.Thesaurus import Thesaurus

def install_configlets( portal, out):
    """
    Method to install platecom user properties...
        @type: portal: PloneSite
        @type out: StringIO

        @rtype: StringIO
        @return: Messages from the GS process
    """
    configTool=getToolByName(portal,'portal_controlpanel',None)
    if configTool:
        for conf in CONFIGLETS:
            configTool.registerConfiglet(**conf)
            out.write('Added configlet %s\n' % conf['id'])

    return out

def install_dependencies( self ):
    """
    Test if everything at DEPENDENCIES gets installed...
        >>> 1
        1
    """
    # If the config contains a list of dependencies, try to install
    # them.  Add a list called DEPENDENCIES to your custom
    # AppConfig.py (imported by config.py) to use it.
    out = StringIO()
    portal = getToolByName(self, 'portal_url').getPortalObject()
    quickinstaller = portal.portal_quickinstaller
    for dependency in DEPENDENCIES:
        print >> out, "Installing dependency %s:" % dependency
        quickinstaller.installProduct(dependency)

    return out

def install( self ):
    """
    External module to install the product...
        @type self: PloneSite
        @param self: The Plone site object

        @rtype: StringIO
        @return: Messages from the install process

    some tests here...
        >>> from icsemantic.thesaurus.config import *
        >>> qi = self.portal.portal_quickinstaller
        >>> installed = [ prod['id'] for prod in qi.listInstalledProducts() ]
        >>> PACKAGENAME in installed
        True

    """
    
    out = StringIO()
    print >> out, "Installing Dependencies"
    install_dependencies(self)

    setup_tool = getToolByName(self, 'portal_setup')
    if shasattr(setup_tool, 'runAllImportStepsFromProfile'):
        # Plone 3
        pass
    else:
        # Plone 2.5.  Would work on 3.0 too, but then it gives tons of
        # DeprecationWarnings when running the tests, causing failures
        # to drown in the noise.
        pass

    install_thesaurus_utility(self, out)

    install_subskin(self, out, GLOBALS)

    install_resources(self, out)

    return out.getvalue()

def install_thesaurus_utility(self, out):
    portal = getToolByName(self, 'portal_url').getPortalObject()
    sm = portal.getSiteManager()
    if not sm.queryUtility(IThesaurus, name=''):
	try:
            sm.registerUtility(Thesaurus(),IThesaurus,'')
	except:
            sm.registerUtility(IThesaurus,Thesaurus(),'')
    
    out.write("Adding Thesaurus Utility\n")

def install_resources(self, out):
    print >> out, "Adding incremental search widget css and kss..."
    css_all = [ 'portal_skins/icsemantic.thesaurus/incrementalSearchSelectWidget.css' ]
    kss_all = [ 'portal_skins/icsemantic.thesaurus/incrementalSearchSelectWidget.kss' ]
    csstool = getToolByName(self, 'portal_css')
    for css in css_all:
        csstool.manage_addStylesheet(
            id = css,
            rel = 'stylesheet',
            rendering = 'link',
            enabled = True,
            cookable = True,
            )
    for kss in kss_all:
        csstool.manage_addStylesheet(id=kss,
            rel='kukit',
            rendering = 'link',
            enabled=True,
            cookable=False,
            )
    print >> out, "Added."

def uninstall( self ):
    """
    Test if everything gets installed...
        >>> 1
        1
    """
    out = StringIO()

    print >> out, "Uninstalling"

    return out.getvalue()



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: admin.py 262 2008-06-12 21:14:31Z flarumbe $
#
# end: Platecom header
"""
admin setting and preferences
Solo vistas y forms

@author: Juan Pablo Gimenez
@contact: jpg@rcom.com.ar
"""
__author__ = """Juan Pablo Gimenez <jpg@rcom.com.ar>"""
__docformat__ = 'plaintext'

import os
from datetime import datetime

import zope
from zope import component
from zope.component import getUtility
from zope.formlib import form
from zope.app.form.browser import MultiSelectSetWidget
from zope.app.form.browser.itemswidgets import MultiSelectWidget \
        as BaseMultiSelectWidget, DropdownWidget, SelectWidget
from zope.app.form.browser import FileWidget

try:
    from zope.lifecycleevent import ObjectModifiedEvent
except:
    from zope.app.event.objectevent import ObjectModifiedEvent

from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.Five.browser import BrowserView
from Products.Five.formlib import formbase

from icsemantic.core import pkg_home
from icsemantic.core.i18n import _
from icsemantic.core.vocabularies import LanguagesVocabularyFactory
from icsemantic.core.browser.base import BaseSettingsForm
from icsemantic.thesaurus import interfaces
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.thesaurus.browser.incrementalSearchSelectWidget import IncrementalSearchSelectWidgetFactory
from azaxview import AzaxView

class ThesaurusUpload( BaseSettingsForm ):
    """ Configlet for upload a thesaurus rdf file.
    """
    form_name = _(u'Thesaurus upload')
    form_fields = form.Fields( interfaces.IicSemanticManagementThesaurusUpload )
    form_fields['thesaurus_file'].custom_widget = FileWidget

    @form.action(_("Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        if form.applyChanges(self.context,
                             self.form_fields,
                             data,
                             self.adapters):
            self.load_thesaurus(data['thesaurus_file'],
			    data['default_language'],
			    contexts=[data['thesaurus_context']],
			    format=data['thesaurus_format'],
			    encoding=data['encoding'],
			    new=data['new'])

            zope.event.notify(
                ObjectModifiedEvent(self.context)
                )
            self.status = _(
                "Updated on ${date_time}",
                mapping={'date_time': str(datetime.utcnow())}
                )
        else:
            self.status = _('No changes')

    def load_thesaurus(self, file_content, default_language,
		    contexts=[], format='SKOSCore', encoding='utf-8',
		    new=False):
        # TODO: add a field Language
        thesaurus_utility().load_string(file_content, default_language,
			contexts=contexts, format=format, encoding=encoding,
			new=new)

class ThesaurusDownload( BrowserView ):
    """ Download thesaurus and query concepts as an RDF File.
    """
    def rdf_string(self):
        return thesaurus_utility().rdf_string()

    def rdf_term_concepts(self):
        return thesaurus_utility().rdf_term_concepts(self.term_parameter(), self.context_parameter())

    def rdf_concepts_search(self):
        return thesaurus_utility().rdf_concepts_search(self.search_parameter(), self.context_parameter())

    def context_parameter(self):
        if self.request.has_key('context'):
            return self.request['context']
        else:
            return None

    def term_parameter(self):
        return self.request['term']

    def search_parameter(self):
        return self.request['search']

class ThesaurusQuery( BrowserView ):
    """ Query the thesaurus.
    """

    def query_result(self):
        return self.azaxview().query_result(self.paging_begin_index(), self.paging_end_index())
    
    def paging_header(self):
        atFirstPage = self.paging_begin_index() == 0
        atLastPage = self.paging_end_index() == len(self.query_result_terms())
        if atFirstPage and atLastPage:
            html = "All results"
        else:
            html = "Page: <select id=\"pageRange\" name=\"pageRange\">" + self.html_page_options() + "</select>"
        return html

    def newTermLanguage(self):
        return "<select id=\"newTermLanguage\" name=\"newTermLanguage\">" + self.html_newTermLanguage_options() + "</select>"
    
    def html_page_options(self):
        html = ""
        for page in range(1, self.last_page()+1):
            html += "<option value=\"%d:%d\"" % (self.paging_begin_index_of(page), self.paging_end_index_of(page))
            if page == self.page():
                html += " selected"
            html += ">%d</option>" % page
        return html
    
    def html_newTermLanguage_options(self):
        html = ""
        for language in LanguagesVocabularyFactory(self.context):
            html += "<option value=\"%s\"" % language.value
            if language.value == self.defaultLanguage():
                html += " selected"
            html += ">%s</option>" % language.title
        return html

    def defaultLanguage(self):
        # TODO: usar utility icsemantic.core.languages.PlatecomPreferredLanguage
        return "es"
    
    def paging_range_of(self, page):
        if self.listIsPaged():
            elementsbypage = self.elementsByPage()
            begin_index = (page - 1) * elementsbypage
            end_index = min(begin_index + elementsbypage, len(self.query_result_terms()))
        else:
            begin_index = 0
            end_index = len(self.query_result_terms())
        return (begin_index, end_index)

    def paging_range(self):
        return paging_range_of(self.page())

    def paging_begin_index(self):
        return self.paging_begin_index_of(self.page())
    
    def paging_end_index(self):
        return self.paging_end_index_of(self.page())

    def paging_begin_index_of(self, page):
        return self.paging_range_of(page)[0]
    
    def paging_end_index_of(self, page):
        return self.paging_range_of(page)[1]
    
    def last_page(self):
        if self.listIsPaged():
            page = (len(self.query_result_terms()) - 1) // self.elementsByPage() + 1
        else:
            page = 1
        return page

    def listIsPaged(self):
        return not self.has_parameter('showall')
    
    def elementsByPage(self):
        if self.has_parameter('elementsbypage'):
            return int(self.request['elementsbypage'])
        else:
            return 20
    
    def page(self):
        if self.has_parameter('page'):
            return int(self.request['page'])
        else:
            return 1

    def has_parameter(self, parameter):
        return self.request.has_key(parameter) and self.request[parameter] != ""
    
    def query_result_terms(self):
        return self.azaxview().query_result_terms()
    
    def azaxview(self):
        if not hasattr(self, "_azaxview"):
            self.setAzaxview(AzaxView(self.context, self.request))
        return self._azaxview
    
    def setAzaxview(self, azaxview):
        self._azaxview = azaxview


class VerticalSelectTest( BaseSettingsForm ):
    """ Test the incremental search select widget.
    """
    form_name = _(u'Vertical Select Test Form')
    form_fields = form.Fields( interfaces.IicSemanticVerticalSelectTest )
    form_fields['vertical_select'].custom_widget = IncrementalSearchSelectWidgetFactory

    @form.action(_("Apply"), condition=form.haveInputWidgets)
    def handle_edit_action(self, action, data):
        if form.applyChanges(self.context,
                             self.form_fields,
                             data,
                             self.adapters):
            zope.event.notify(
                ObjectModifiedEvent(self.context)
                )
            self.status = _(
                "Updated on ${date_time}.\nConcepts selected: ${selected_concepts}",
                mapping={
                    'date_time': str(datetime.utcnow()),
                    'selected_concepts': str(data['vertical_select'])
                }
                )
        else:
            self.status = _('No changes')



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: azaxview.py 262 2008-06-12 21:14:31Z flarumbe $
#
# end: Platecom header

try:
    from azax import AzaxBaseView, force_unicode
    from azax.plugins.core.interfaces import IKSSCoreCommands
    from azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
except ImportError:
    try:
        from Products.azax import AzaxBaseView, force_unicode
        from Products.azax.plugins.core.interfaces import IKSSCoreCommands
        from Products.azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
    except ImportError:
	from kss.core import force_unicode
        from kss.core.kssview import AzaxBaseView
        from kss.core.plugins.core.interfaces import IKSSCoreCommands
	from kss.core.plugins.effects.interfaces import IScriptaculousEffectsCommands

from Products.CMFPlone.utils import safe_unicode

from pyThesaurus.config import relations
from pyThesaurus.config import relation_name
from pyThesaurus.Concept import Concept
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.core.vocabularies import LanguagesVocabularyFactory
import icsemantic.core.browser.admin

import cgi

class AzaxView(AzaxBaseView):

    #header_macros = ViewPageTemplateFile('browser/header_macros.pt')

    def openConcept(self, cid, openedConcepts):
        if not cid in openedConcepts.split(","):
            self.setOpenedConcepts(openedConcepts)
            t = thesaurus_utility()
            int_cid = int(cid)
            concept = t[int_cid]
            term = self.preferedTerm(concept)
            self.insertHTMLConcept(id=cid, term=term, cid=int_cid, concept=concept)
            self.updateOpenedConcepts()
        return self.render()
    
    def addTerm(self, term, language, openedConcepts):
        self.setOpenedConcepts(openedConcepts)
        newterm = term + "@" + language
        if not newterm in self.openedTerms():
            self.insertHTMLConcept(id=newterm, term=newterm)
            self.updateOpenedConcepts()
        return self.render()

    def saveConcepts(self, conceptsRelations, openedConcepts):
        self.setOpenedConcepts(openedConcepts)
        self.emptyConceptsRelations(self.termsRelationsList(conceptsRelations))
        self.appendTermsToConceptsRelations(self.termsRelationsList(conceptsRelations))
        self.appendNewConcepts()
        self.replaceNewConceptsHTML()
        thesaurus_utility().commit()
        self.updateOpenedConcepts()
        #print conceptsRelations
        #IKSSCoreCommands(self).insertHTMLAsLastChild('div#dhtmlgoodies_mainContainer', "GRABAR: " + conceptsRelations)
        return self.render()
    
    def viewPage(self, pageRange, context, term, search, languages, format):
        self.setRequestIfNotEmpty('context', context)
        self.setRequestIfNotEmpty('term', term)
        self.setRequestIfNotEmpty('search', search)
        self.setRequestIfNotEmpty('languages', languages)
        self.setRequestIfNotEmpty('format', format)
        begin_index = int(pageRange.split(":")[0])
        end_index = int(pageRange.split(":")[1])
        IKSSCoreCommands(self).replaceInnerHTML('ul#allItems', self.query_result(begin_index, end_index))
        return self.render()
    
    def conceptsSearch(self, context, term, search, languages, format, elementsByPage, showall):
        self.setRequestIfNotEmpty('context', context)
        self.setRequestIfNotEmpty('term', term)
        self.setRequestIfNotEmpty('search', search)
        self.setRequestIfNotEmpty('languages', languages)
        self.setRequestIfNotEmpty('format', format)
        self.setRequestIfNotEmpty('elementsByPage', elementsByPage)
        self.setRequestIfNotEmpty('showall', showall)
        begin_index = self.paging_begin_index()
        end_index = self.paging_end_index()
        IKSSCoreCommands(self).replaceInnerHTML('ul#allItems', self.query_result(begin_index, end_index))
        IKSSCoreCommands(self).replaceHTML('input#currentSearchExpression', u'<input type="hidden" name="currentSearchExpression" id="currentSearchExpression" value="' + cgi.escape(search) + '">')
        IKSSCoreCommands(self).replaceInnerHTML('span#pagingHeader', self.paging_header())
        return self.render()

    def insertHTMLConcept(self, id, term, cid=None, concept=None):
        term_box = self.html_term_box(id, cid, concept, term)
        IKSSCoreCommands(self).insertHTMLAsLastChild('div#dhtmlgoodies_mainContainer', u'<div id="box_%s">%s</div>' % (safe_unicode(id), term_box))
        self.newOpenedConcept(id)

    def replaceHTMLConcept(self, newterm, cid):
        t = thesaurus_utility()
        id = repr(cid)
        concept = t[cid]
        term = self.preferedTerm(concept)
        term_box = self.html_term_box(id, cid, concept, term)

        IKSSCoreCommands(self).replaceHTML('div#box_%s' % safe_unicode(newterm), u'<div id="box_%s">%s</div>' % (safe_unicode(id), term_box))
        self.newOpenedConcept(id)
    
    def html_term_box(self, id, cid, concept, term):
        term_box = "<p align=\"center\">%s</p>" % safe_unicode(term)
        for r in relations:
            term_box += self.html_relation_div(id, r, cid, concept, term)
        return term_box
    
    def html_relation_div(self, id, r, cid, concept, term):
        relation_id = u"%s_%s" % (safe_unicode(id), cgi.escape(safe_unicode(r)))
        html_relation_div = u"<div>\n\t\t\t\t\t<p>" + cgi.escape(safe_unicode(relation_name[r])) + u":</p>"
        html_relation_div += u"\n\t\t\t\t\t<ul id=\"relation_" + relation_id + u"\">"
        html_relation_div += self.all_list_item(relation_id)
        if cid is not None:
            html_relation_div += self.related_terms(concept[r], cid, r)
        elif r == '=':
            html_relation_div += self.term_list_item(term, -1, r)
        html_relation_div += u"\n\t\t\t\t\t</ul>"
        html_relation_div += u"\n\t\t\t\t</div>"
        return html_relation_div

    def openedTerms(self):
        t = thesaurus_utility()
        terms = []
        for id in self.openedConcepts():
            if id.find('@') != -1:
                terms.append(id)
            elif id != 'x':
                terms.append(self.preferedTerm(t[int(id)]))
        return terms
    
    def setOpenedConcepts(self, openedConcepts):
        self._openedConcepts = openedConcepts
    
    def newOpenedConcept(self, id):
        self._openedConcepts += "," + id

    def openedConcepts(self):
        return self._openedConcepts.split(',')
    
    def updateOpenedConcepts(self):
        IKSSCoreCommands(self).replaceHTML('input#openedConcepts', u'<input type="hidden" id="openedConcepts" name="openedConcepts" value="' + cgi.escape(safe_unicode(self._openedConcepts)) + '">')
    
    def paging_begin_index(self):
        return self.thesaurusQuery().paging_begin_index()
    
    def paging_end_index(self):
        return self.thesaurusQuery().paging_end_index()

    def query_result(self, begin_index, end_index):
        html = ""
        if self.format() == "draganddrop":
            html = self.all_list_item()
        for (term, cid) in self.query_result_terms()[begin_index:end_index]:
            if self.format() == "simple":
                html += term + "<br>"
            else:
                html += self.term_list_item(term, cid)
        return html

    def query_result_concepts(self):
        if not hasattr(self, "_query_result_concepts"):
            if not self.has_parameter('context') and not self.has_parameter('term') and not self.has_parameter('search'):
                self._query_result_concepts = self.concepts()
            elif self.has_parameter('term'):
                self._query_result_concepts = self.term_concepts()
            elif self.has_parameter('search'):
                self._query_result_concepts = self.concepts_search()
            else:
                raise Exception, "Incorrect parameters."
        return self._query_result_concepts
    
    def query_result_terms(self):
        if not hasattr(self, "_query_result_terms"):
            t = thesaurus_utility()
            self._query_result_terms = [ (self.preferedTerm(t[cid]), cid) for cid in self.query_result_concepts() if len(t[cid].get_prefered(self.languages())) > 0 ]
        return self._query_result_terms

    def concepts(self):
        return thesaurus_utility().concepts()

    def term_concepts(self):
        return thesaurus_utility().term_concepts_ids(self.term_parameter(), self.context_parameter())

    def concepts_search(self):
        return thesaurus_utility().concepts_search_ids(self.search_parameter(), self.context_parameter())

    def languages(self):
        if self.has_parameter('languages'):
            return self.request['languages'].split(",")
        else:
            return [ language.value for language in LanguagesVocabularyFactory(self.context) ]
    
    def format(self):
        if self.has_parameter('format'):
            return self.request['format']
        else:
            return "draganddrop"
    
    def has_parameter(self, parameter):
        return self.request.has_key(parameter) and self.request[parameter] != ""

    def context_parameter(self):
        if self.has_parameter('context'):
            return self.request['context']
        else:
            return None

    def term_parameter(self):
        return self.request['term']

    def search_parameter(self):
        return self.request['search']
    
    def setRequestIfNotEmpty(self, parameter, value):
        if value != "":
            self.request.set(parameter, value)

    def appendTermsToConceptsRelations(self, termsRelations):
        """
        It appends terms of relations of expanded concepts that appear in the page.
        """
        for (concept, r, term) in termsRelations:
            concept[r].append(term)

    def emptyConceptsRelations(self, termsRelations):
        """
        It empties concepts expanded by the user relations.
        """
        for (concept, r, term) in termsRelations:
            for r in relations:
                concept[r] = list()
    
    def appendNewConcepts(self):
        """
        It appends valid new concepts created by user.
        """
        for term, concept in self.newConcepts().items():
            if term in concept['=']:
                cid = self.appendNewConcept(concept)
                self.newConcepts()[term] = cid # OJO: The concept is replaced by its cid

    def appendNewConcept(self, concept):
        return thesaurus_utility().append_concept(concept)

    def replaceNewConceptsHTML(self):
        """
        Now the new concepts are opened concept.
        """
        for term, cid in self.newConcepts().items():
            if type(cid) == type(1):
                self.replaceHTMLConcept(term, cid)
    
    def termsRelationsList(self, conceptsRelations):
        """
        Returns a list of (concept, relationSymbol, term) that appear expanded in the page.
        """
        ret = []
        for relationAndTerm in conceptsRelations.split(";"):
            if not relationAndTerm.startswith("allItems") and not relationAndTerm.startswith("selectedItems"):
                relation = relationAndTerm.split("|")[0]
                term = relationAndTerm.split("|")[1].split("_")[0]
                if term != "all":
                    id = relation.split("_")[1]
                    r = relation.split("_")[2]
                    ret.append((self.concept(id), r, term))
        return ret
    
    def concept(self, id):
        try:
            cid = int(id)
            aConcept = thesaurus_utility()[cid]
        except:
            aConcept = self.newConcept(id)
        return aConcept
    
    def newConcept(self, term):
        if not term in self.newConcepts():
            self.newConcepts()[term] = Concept(et=[term])
        return self.newConcepts()[term]
    
    def newConcepts(self):
        if not hasattr(self, "_newConcepts"):
            self._newConcepts = dict()
        return self._newConcepts
    
    def related_terms(self, terms, cid, r):
        html = ""
        for term in terms:
            html += self.term_list_item(term, cid, r)
        return html

    def all_list_item(self, box_id="results"):
        return u"\n\t\t\t\t\t\t<li id=\"all_" + box_id + u"\"><img src=\"fullscreenexpand_icon.gif\" border=\"0\" id=\"dragimage_all_" + box_id + u"\"> All</li>"

    def term_list_item(self, term, cid, r=""):
        id = u"%s_%d" % (cgi.escape(safe_unicode(term)), cid)
        if r != "":
            id += u"-copy%s" % cgi.escape(r)
        return u"\n\t\t\t\t\t\t<li id=\"" + id + u"\"><img src=\"search_icon.gif\" border=\"0\" id=\"expandimage_" + id + "\" class=\"open-concept\" kukit:cid=\"" + repr(safe_unicode(cid)) + u"\"><img src=\"fullscreenexpand_icon.gif\" border=\"0\" id=\"dragimage_" + id + u"\">" + cgi.escape(safe_unicode(term)) + u"</li>"

    def newterm_list_item(self, term):
        id = u"%s_%d-copy%d" % (safe_unicode(term), -1, self.newTermCopyId())
        return u"\n\t\t\t\t\t\t<li id=\"" + id + u"\"><img src=\"fullscreenexpand_icon.gif\" border=\"0\" id=\"dragimage_" + id + u"\">" + safe_unicode(term) + u"</li>"
    
    def newTermCopyId(self):
        if not hasattr(self, "_newTermCopyId"):
            _newTermCopyId = 0
        _newTermCopyId -= 1
        return _newTermCopyId
    
    def paging_header(self):
        return self.thesaurusQuery().paging_header()
    
    def thesaurusQuery(self):
        if not hasattr(self, "_thesaurusQuery"):
            self.setThesaurusQuery(icsemantic.thesaurus.browser.admin.ThesaurusQuery(self.context, self.request))
            self._thesaurusQuery.setAzaxview(self)
        return self._thesaurusQuery
    
    def setThesaurusQuery(self, thesaurusQuery):
        self._thesaurusQuery = thesaurusQuery

    def preferedTerm(self, concept):
        return concept.get_prefered(self.languages())[0]



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: __init__.py 258 2008-06-12 15:57:43Z crocha $
#
# end: Platecom header
""" icsemantic.thesaurus interfaces.
"""

from zope import schema
from zope.interface import Interface
from icsemantic.core.i18n import _

class IThesaurus( Interface ):
    """ Interface for Thesaurus Utility
    """

class IicSemanticManagementThesaurusUpload( Interface ):
    """ Interface used for a thesaurus file upload.
    """

    thesaurus_file = schema.Bytes(title = _(u"Thesaurus file"),
                                 required = True,
                                 description = _(u"A thesaurus file located in your computer"))

    default_language = schema.Choice(
		    title = _(u"heading_thesaurus_language",
			    default=u"Default language"),
		    description = _(u"description_site_language",
			    default=u"If term have no language is defined as this language"),
		    required = True,
		    vocabulary="icsemantic.available_languages")
    thesaurus_context = schema.TextLine(
		    title = _(u"heading_thesaurus_context",
			    u"Default Context"),
		    description = _(u"description_thesaurus_context",
			    default=u"If term have no context is defined as this context"),
                    required = False) #When updated will be required
    thesaurus_format = schema.Choice(
		    title = _(u"heading_file_format",
			    u"File format"),
		    description = _(u"description_file_format",
			    u"Thesaurus File format"),
		    required = True,
                    vocabulary="icsemantic.core.thesaurus_formats") #When updated will be required
    encoding = schema.Choice(
		    title = _(u"heading_encodings",
			    u"Encoding"),
		    description = _(u"description_encodings",
			    u"Encoding File format"),
		    required = True,
                    vocabulary="icsemantic.core.encodings")
    new = schema.Bool(
		    title = _(u"heading_new",
			    u"Start new thesaurus"),
		    description = _(u"description_new",
			    u"Clean old thesaurus and create one with this data."),
		    required = True,
		    )

class IicSemanticVerticalSelectTest( Interface ):
    """ Interface used for test the vertical select widget.
    """
    vertical_select = schema.TextLine(
		    title = _(u"heading_thesaurus_verticalselect",
			    u"Vertical select"),
		    description = _(u"description_thesaurus_verticalselect",
			    default=u"Select a branch of concepts"),
            required = False)




# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: Thesaurus.py 261 2008-06-12 21:09:24Z flarumbe $
#
# end: Platecom header

from zope.interface import implements
from interfaces import IThesaurus
from pyThesaurus.Thesaurus import Thesaurus as NonPersistentThesaurus
from OFS.SimpleItem import SimpleItem
import pyThesaurus.ioSKOSCore as ioSKOSCoreModule
from pyThesaurus.ioSKOSCore import ioSKOSCore
from pyThesaurus.ioDing import ioDing
import transaction
import StringIO
from zope.app.component.hooks import getSite

from persistent import Persistent, list, dict

_format = {
	'SKOSCore': ioSKOSCore,
	'Ding': ioDing,
	}

class Thesaurus(SimpleItem):
    implements(IThesaurus)

    def __init__(self):
        self._thesaurus = NonPersistentThesaurus()
        wrap(self, "_thesaurus")
        self.commit()

    def clean(self):
	self._thesaurus = NonPersistentThesaurus()
        self.commit()
    
    def load(self, filename, lang, contexts=[], format='SKOSCore', encoding='utf-8', new=False):
	if new: self._thesaurus = NonPersistentThesaurus()
	io = _format[format](lang, contexts=contexts, thesaurus=self._thesaurus)
        file = open(filename)
        io.read(file, encoding=encoding)
	file.close()
        self.commit()
    
    def load_string(self, string, lang, contexts=[], format='SKOSCore', encoding='utf-8', new=False):
	if new: self._thesaurus = NonPersistentThesaurus()
	io = _format[format](lang, contexts=contexts, thesaurus=self._thesaurus)
	file = StringIO.StringIO(string)
        io.read(file, encoding=encoding)
	file.close()
        self.commit()
    
    def save(self, filename, lang='', contexts=[], format='SKOSCore', encoding='utf-8'):
	io = _format[format](lang, contexts=contexts, thesaurus=self._thesaurus)
        file = open(filename, 'w')
	io.write(file, encoding=encoding)
	file.close()

    def rdf_string(self, lang='', contexts=[], format='SKOSCore', encoding='utf-8'):
	io = _format[format](lang, contexts=contexts,
			thesaurus=self._thesaurus)
        temp = StringIO.StringIO()
	io.write(temp)
        return temp.getvalue()

    def rdf_term_concepts(self, term, context=None, encoding='utf-8'):
        temp = StringIO.StringIO()
        ioSKOSCoreModule.write_concepts(temp,
			self.term_concepts(term, context))
        return temp.getvalue()

    def rdf_concepts_search(self, search_expression, context=None):
        temp = StringIO.StringIO()
        ioSKOSCoreModule.write_concepts(temp,
			self.concepts_search_objects(search_expression, context))
        return temp.getvalue()

    def commiting_methods(self):
        return [ "set_prefered", "append_term", "append_concept", "replace_concept_at" ]

    def commit(self):
        t = self._thesaurus
        self._thesaurus = t
        transaction.commit()
    
def wrap(anObject, attribute_name):
    """
    Makes this class wrap an attribute. It adds to this class the methods of the wrapped object and forwards to it. It also execute commit method after calling the wrapped one.
    """
    if not hasattr(anObject.__class__, attribute_name + "_wrapped") or \
        not getattr(anObject, attribute_name + "_wrapped"):
        wrapped_class = eval("anObject." + attribute_name).__class__
	n_items = vars(wrapped_class).items()
	for inhe_class in wrapped_class.__bases__:
	    n_items += vars(inhe_class).items()
        for func_name, func in n_items:
            if hasattr(func, "func_code") and (not func_name.startswith("_") or func_name == "__getitem__" or func_name == "__call__"):
                # Parameters for declaring and for invoking
                pars_declare = []
                pars_invoke = []
                for i in range(func.func_code.co_argcount):
                    par = func.func_code.co_varnames[i]
                    if i > 0:
                        pars_invoke.append(par)
                    
                    if func.func_defaults != None:
                        default_index = i - (func.func_code.co_argcount - len(func.func_defaults))
                        if default_index >= 0:
                            par += "=" + repr(func.func_defaults[default_index])
                    pars_declare.append(par)
                
                # Function code
                code = "def " + func_name + "(" + ", ".join(pars_declare) + "): "
                if func_name in anObject.commiting_methods():
                    code += "tmp = self." + attribute_name + "." + func_name + "("
                    code += ", ".join(pars_invoke) + "); self.commit(); return tmp"
                else:
                    code += "return self." + attribute_name + "." + func_name + "("
                    code += ", ".join(pars_invoke) + ");"

                #print code + "\n"
                exec(code)
                setattr(anObject.__class__, func_name, eval(func_name))
        setattr(anObject.__class__, attribute_name + "_wrapped", True)

def thesaurus_utility():
    from zope.component import queryUtility
    ut = queryUtility(IThesaurus)
    if ut is None:
        raise NameError, "Thesaurus utility does not exist. Please, reinstall icsemantic.thesaurus product."
    wrap(ut, "_thesaurus")
    return ut




# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: test_related_items.py 292 2008-06-17 17:57:20Z flarumbe $
#
# end: Platecom header
import unittest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase.layer import PloneSite

from icsemantic.thesaurus.config import *

from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase.setup import portal_name
from icsemantic.thesaurus.interfaces import IThesaurus

from AccessControl.SecurityManagement import newSecurityManager

import base

class TestRelatedItems(base.icSemanticTestCase):

    def warning(self, err):
        import sys
        sys.stderr.write("\nWarning: %s\n" % err)

    def error(self, err):
        import sys
        sys.stderr.write("\nERROR: %s\n" % err)

    def assertEqualSets(self, a, b):
        a.sort()
        b.sort()
        self.assertEqual(a, b)

    def testRelatedItems(self):
        doc1 = self.addDocument("doc1", "A document with wetness keyword", ["wetness"])
        doc2 = self.addDocument("doc2", "A document with dryness keyword", ["dryness"])
        doc3 = self.addDocument("doc3", "A document with no keywords", [])
        assert not self.documentIsRelatedTo(doc1, doc1)
        assert self.documentIsRelatedTo(doc1, doc2)
        assert not self.documentIsRelatedTo(doc1, doc3)
        assert self.documentIsRelatedTo(doc2, doc1)
        assert not self.documentIsRelatedTo(doc2, doc2)
        assert not self.documentIsRelatedTo(doc2, doc3)
        assert not self.documentIsRelatedTo(doc3, doc1)
        assert not self.documentIsRelatedTo(doc3, doc2)
        assert not self.documentIsRelatedTo(doc3, doc3)

    def addDocument(self, document_id, document_title, keywords):
        self.folder[self.thisTestFolderName()].invokeFactory("Document", document_id)
        doc = getattr( self.folder[self.thisTestFolderName()], document_id)
        doc.setTitle(document_title)
        doc.setSubject(keywords)
        doc.reindexObject()
        return doc

    def documentIsRelatedTo(self, aDocument, otherDocument):
        return otherDocument in aDocument.computeRelatedItems()

    def afterSetUp(self):
        self.login()
        self.setRoles(['Manager', 'Member'])
        """if self.isInstalled("icsemantic.core"):
            self.uninstall("icsemantic.core")
        if self.isInstalled("icsemantic.thesaurus"):
            self.uninstall("icsemantic.thesaurus")
        if self.isInstalled("icsemantic.catalog"):
            self.uninstall("icsemantic.catalog")

        self.install("icsemantic.thesaurus")
        self.install("icsemantic.catalog")
        self.install("icsemantic.core")"""
        if not self.isInstalled("icsemantic.core"):
            self.install("icsemantic.core")
        if not self.isInstalled("icsemantic.thesaurus"):
            self.install("icsemantic.thesaurus")
        if not self.isInstalled("icsemantic.catalog"):
            self.install("icsemantic.catalog")

        self.loadThesaurus()
        self.prepareEmptyFolder(self.thisTestFolderName())

    def thisTestFolderName(self):
        return "icsemantic.core.relatedItems"

    def loadThesaurus(self):
        self.thesaurus_utility().load("src/pyThesaurus/pyThesaurus/tests/data/sample001.rdf", "en")

    def install(self, product_name):
        return self.qi().installProduct(product_name)

    def uninstall(self, product_name):
        return self.qi().uninstallProducts((product_name,))

    def isInstalled(self, product_name):
        return self.qi().isProductInstalled(product_name)

    def qi(self):
        return self.portal.portal_quickinstaller

    def prepareEmptyFolder(self, folder_name):
        if not folder_name in self.folder.objectIds():
            self.folder.invokeFactory('Folder', id=folder_name)
        for id in self.folder[folder_name].objectIds():
            self.folder[folder_name].manage_delObjects(id)

    def thesaurus_utility(self):
        sm = self.portal.getSiteManager()
        ut = sm.utilities.queryUtility(IThesaurus)
        if ut is None:
            raise NameError, "Thesaurus utility does not exist. Please, reinstall icsemantic.thesaurus product."
        return ut

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestRelatedItems))
    return suite



try:
    from azax import AzaxBaseView, force_unicode
    from azax.plugins.core.interfaces import IKSSCoreCommands
    from azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
except ImportError:
    try:
        from Products.azax import AzaxBaseView, force_unicode
        from Products.azax.plugins.core.interfaces import IKSSCoreCommands
        from Products.azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
    except ImportError:
	from kss.core import force_unicode
        from kss.core.kssview import AzaxBaseView
        from kss.core.plugins.core.interfaces import IKSSCoreCommands
	from kss.core.plugins.effects.interfaces import IScriptaculousEffectsCommands

from Products.CMFPlone.utils import safe_unicode

from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.core.vocabularies import LanguagesVocabularyFactory
import icsemantic.core.browser.admin

import cgi

class AutoComplete(AzaxBaseView):

    def autocompleteConcepts(self, searchExpression):
        #TODO: probar .setAttribute
        #import ipdb
        #ipdb.set_trace()
        IKSSCoreCommands(self).replaceInnerHTML('div#searchResults', 'Resultado de "' + cgi.escape(safe_unicode("<br>".join(searchExpression.split(",")))) + '"')
        return self.render()



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: config.py 306 2008-06-24 19:40:34Z flarumbe $
#
# end: Platecom header
import icsemantic.thesaurus
PROJECTNAME = "icsemantic.thesaurus"
PACKAGE = icsemantic.thesaurus
PACKAGENAME = "icsemantic.thesaurus"
DEPENDENCIES = [ 'icsemantic.core', 'icsemantic.langfallback', ]

GLOBALS = globals()



"""Monkey patches
"""

from Products.LinguaPlone.I18NBaseObject import I18NBaseObject

from icsemantic.thesaurus.Thesaurus import thesaurus_utility

def get_translations(portal, k, lang=None):
    """Query the local thesaurus for translated concepts"""
    try:
        r = thesaurus_utility().get_equivalent(k, lang, exclude=False)
    except IndexError:
        r = []
    return r

def addTranslation(self, language, *args, **kwargs):
    """Do what Products.LinguaPlone.I18NBaseObject.addTranslation does
    but also add translations of the keywords to the translated
    version.
    """
    self.old_addTranslation(language, *args, **kwargs)
    canonical = self.getCanonical()
    translation = self.getTranslation(language)
    subject = canonical.Subject()
    translated_subject = []

    portal = self.portal_url.getPortalObject()

    for s in subject:
        translated_subject += get_translations(portal, \
            s+'@'+self.getCanonicalLanguage(), \
            lang=[language])

    # Remove the @lang part
    translated_subject = map(lambda x: x[:-(len(language)+1)],
                             translated_subject)

    translation.setSubject(translated_subject)
    translation.reindexObject()


I18NBaseObject.old_addTranslation = I18NBaseObject.addTranslation
I18NBaseObject.addTranslation = addTranslation



"""Tests
"""
import unittest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc
from Products.PloneTestCase import PloneTestCase as ptc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase.layer import PloneSite

from icsemantic.thesaurus.config import *
import base

class FunctionalTestConfiglets(base.icSemanticFunctionalTestCase):
    """
    """

def test_suite():
    return unittest.TestSuite([

        # Integration tests that use PloneTestCase
        ztc.FunctionalDocFileSuite(
            'test_addtranslation_patch.txt',
            package=PACKAGENAME + '.tests',
            test_class=FunctionalTestConfiglets),

        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: test_thesaurus_download.py 261 2008-06-12 21:09:24Z flarumbe $
#
# end: Platecom header
import unittest
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.thesaurus.browser.admin import ThesaurusQuery
from icsemantic.thesaurus.browser.azaxview import AzaxView
from zope.app.component.hooks import setSite

import base

class TestThesaurusAdmin(base.icSemanticTestCase):

    def testSaveConceptsRelations(self):
        conceptsRelations = u"allItems|all_results;allItems|abarraganamiento@es_61;allItems|abarraganarse@es_62;allItems|adulterio@es_418;allItems|adúltero@es_426;allItems|aguacharse@es_531;allItems|ajuntarse@es_605;allItems|amanerado@es_828;allItems|amanerado@es_829;allItems|amanerar@es_830;allItems|amanerar@es_831;allItems|amansar@es_832;allItems|amansar@es_833;allItems|amante@es_834;allItems|amante@es_835;allItems|amanuense@es_836;allItems|barraganería@es_1670;allItems|engolamiento@es_3163;allItems|madrugada@es_6142;allItems|madrugar@es_6143;allItems|melindre@es_6265;selectedItems|all_selected;relation_nuevotermino@es_=|all_nuevotermino@es_=;relation_nuevotermino@es_=|nuevotermino@es_-1-copy=;relation_nuevotermino@es_=|engolamiento@es_3163-copy1;relation_nuevotermino@es_-|all_nuevotermino@es_-;relation_nuevotermino@es_-|barraganería@es_1670-copy2;relation_nuevotermino@es_#|all_nuevotermino@es_#;relation_nuevotermino@es_#|madrugada@es_6142-copy3;relation_nuevotermino@es_<|all_nuevotermino@es_<;relation_nuevotermino@es_>|all_nuevotermino@es_>;relation_nuevotermino@es_!|all_nuevotermino@es_!;relation_nuevotermino@es_~|all_nuevotermino@es_~"
        self.saveConcepts(conceptsRelations)

        table = [
                    ("nuevotermino@es", '=', ["nuevotermino@es", "engolamiento@es"]),
                    ("nuevotermino@es", '-', [u"barraganería@es"]),
                    ("nuevotermino@es", '#', ["madrugada@es"]),
                    ("nuevotermino@es", '<', []),
                    ("nuevotermino@es", '>', []),
                    ("nuevotermino@es", '!', []),
                    ("nuevotermino@es", '~', [])
                ]
        self.conceptsRelationsCorrect(conceptsRelations, table)
    
    def testReadConceptsHTMLBis(self):
        html_downloaded = self.readTermConceptsHTML("ababa@es", ["es"])
        html_expected = "ababa@es<br>ababol@es<br>"
        assert html_downloaded == html_expected
    
    def testReadConceptsHTMLBis(self):
        html_downloaded = self.searchConceptsHTML("#", ["es"])
        html_expected = ""
        assert html_downloaded == html_expected
    
    def testConceptsPagingAll(self):
        html_downloaded = self.searchConceptsHTML("ama", ["es"])
        html_expected = u"ababa@es<br>ababol@es<br>abarraganamiento@es<br>abarraganarse@es<br>acaramelado@es<br>acerbo@es<br>acongojar@es<br>adiestrador@es<br>adorable@es<br>adorar@es<br>adulterio@es<br>adúltero@es<br>afectivo@es<br>aferrar@es<br>aficionado@es<br>agonía@es<br>agrado@es<br>agraz@es<br>agriar@es<br>aguacharse@es<br>aguar@es<br>ajuntarse@es<br>ama@es<br>ama@es<br>amaestrado@es<br>amagar@es<br>amago@es<br>amainar@es<br>amalgama@es<br>amalgamar@es<br>amalgamar@es<br>amanerado@es<br>amanerado@es<br>amanerar@es<br>amanerar@es<br>amansar@es<br>amansar@es<br>amante@es<br>amante@es<br>amanuense@es<br>amar@es<br>amarar@es<br>amargar@es<br>amarillear@es<br>amarras@es<br>amarre@es<br>amarre@es<br>amarrete@es<br>amartillar@es<br>amartillar@es<br>amasar@es<br>amasar@es<br>amazacotado@es<br>amañarse@es<br>amaño@es<br>amigable@es<br>amortiguar@es<br>anclaje@es<br>apañado@es<br>apelmazar@es<br>atado@es<br>atentamente@es<br>barloa@es<br>barraganería@es<br>batiburrillo@es<br>benigno@es<br>bilioso@es<br>bilis@es<br>bita@es<br>bácara@es<br>cabalgador@es<br>cabildear@es<br>dársena@es<br>engolamiento@es<br>erróneo@es<br>erótico@es<br>falso@es<br>foguear@es<br>gentil@es<br>gentileza@es<br>grato@es<br>gualdo@es<br>lactancia@es<br>lactar@es<br>liar@es<br>ligar@es<br>madama@es<br>madrugada@es<br>madrugar@es<br>masaje@es<br>mazacote@es<br>melindre@es<br>molesto@es<br>nana@es<br>nobleza@es<br>quemazón@es<br>valido@es<br>yacer@es<br>zabila@es<br>ámbar@es<br>"
        assert html_downloaded == html_expected
        
    def testAllConceptsPagingPage1(self):
        html_downloaded = self.thesaurusConceptsListHTML(languages=["es"], page="1", elementsbypage="20")
        html_expected = u"Biblia@es<br>Dios@es<br>Jesucristo@es<br>Pelota@es<br>UCI@es<br>Yahvé@es<br>a priori@es<br>a propósito@es<br>a quemarropa@es<br>a ultranza@es<br>ababa@es<br>ababol@es<br>ababol@es<br>abacera@es<br>abacero@es<br>abacería@es<br>abacial@es<br>abacora@es<br>abacorar@es<br>abacá@es<br>"
        assert html_downloaded == html_expected

    def testConceptsPagingPage1(self):
        html_downloaded = self.searchConceptsHTML("ama", ["es"], page="1", elementsbypage="10")
        html_expected = u"ababa@es<br>ababol@es<br>abarraganamiento@es<br>abarraganarse@es<br>acaramelado@es<br>acerbo@es<br>acongojar@es<br>adiestrador@es<br>adorable@es<br>adorar@es<br>"
        assert html_downloaded == html_expected
    
    def testConceptsPagingPage2(self):
        html_downloaded = self.searchConceptsHTML("ama", ["es"], page="2", elementsbypage="10")
        html_expected = u"adulterio@es<br>adúltero@es<br>afectivo@es<br>aferrar@es<br>aficionado@es<br>agonía@es<br>agrado@es<br>agraz@es<br>agriar@es<br>aguacharse@es<br>"
        assert html_downloaded == html_expected
    
    def testConceptsPagingPage10(self):
        html_downloaded = self.searchConceptsHTML("ama", ["es"], page="10", elementsbypage="10")
        html_expected = u"mazacote@es<br>melindre@es<br>molesto@es<br>nana@es<br>nobleza@es<br>quemazón@es<br>valido@es<br>yacer@es<br>zabila@es<br>ámbar@es<br>"
        assert html_downloaded == html_expected
    
    def testPageList(self):
        html_downloaded = self.searchConceptsPageList("aman", ["es"], page="1", elementsbypage="10")
        html_expected = u'Page: <select id="pageRange" name="pageRange"><option value="0:10" selected>1</option><option value="10:20">2</option><option value="20:21">3</option></select>'
        assert html_downloaded == html_expected

        html_downloaded = self.searchConceptsPageList("aman", ["es"], page="2", elementsbypage="10")
        html_expected = u'Page: <select id="pageRange" name="pageRange"><option value="0:10">1</option><option value="10:20" selected>2</option><option value="20:21">3</option></select>'
        assert html_downloaded == html_expected

        html_downloaded = self.searchConceptsPageList("aman", ["es"], page="3", elementsbypage="10")
        html_expected = u'Page: <select id="pageRange" name="pageRange"><option value="0:10">1</option><option value="10:20">2</option><option value="20:21" selected>3</option></select>'
        assert html_downloaded == html_expected

        html_downloaded = self.searchConceptsPageList("aman", ["es"], page="1", elementsbypage="100")
        html_expected = u'Todos los resultados'
        assert html_downloaded == html_expected

    def testOpenConcept(self):
        html_downloaded = self.openConcept(cid="22")

    def testViewPage(self):
        html_downloaded = self.viewPage("1:10", search="aman", languages="es")
    
    def testSearchRequest(self):
        html_downloaded = self.searchRequest(search="ama", languages="es")
    
    def testAddTerm(self):
        html_downloaded = self.addTerm(term="unTermino", language="es")
        html_downloaded = self.addTerm(term="unTermino", language="es", openedConcepts="x,210,unTermino@es")
        html_downloaded = self.addTerm(term="acatarrarse", language="es", openedConcepts="x,210,unTermino@es")
    
    def testSaveNewConcept(self):
        conceptsRelations = "allItems|all_results;allItems|wetness@en_0;allItems|shrubs@en_1;allItems|rocks@en_2;selectedItems|all_selected;relation_2_=|all_2_=;relation_2_=|rocks@en_2-copy=;relation_2_=|slate@en_2-copy=;relation_2_-|all_2_-;relation_2_-|granite@en_2-copy=;relation_2_-|basalt@en_2-copy=;relation_2_#|all_2_#;relation_2_<|all_2_<;relation_2_>|all_2_>;relation_2_!|all_2_!;relation_2_~|all_2_~;relation_unTermino@es_=|all_unTermino@es_=;relation_unTermino@es_=|unTermino@es_unTermino@es_=;relation_unTermino@es_=|otroTermino@es_unTermino@es_="
        html_downloaded = self.saveConcepts(conceptsRelations)

    def openConcept(self, cid, openedConcepts="x"):
        return self.azaxview().openConcept(cid, openedConcepts)

    def viewPage(self, pageRange, context="", term="", search="", languages="", format=""):
        return self.azaxview().viewPage(pageRange, context, term, search, languages, format)

    def searchRequest(self, context="", term="", search="", languages="", format="", elementsByPage="", showall=""):
        return self.azaxview().conceptsSearch(context, term, search, languages, format, elementsByPage, showall)

    def addTerm(self, term, language, openedConcepts="x"):
        return self.azaxview().addTerm(term, language, openedConcepts)

    def thesaurusConceptsListHTML(self, languages=["en"], page="", elementsbypage=""):
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        return browserView.query_result()

    def readTermConceptsHTML(self, term, languages=["en"], page="", elementsbypage=""):
        self.app.REQUEST.set('term', term)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        return browserView.query_result()

    def searchConceptsHTML(self, search_expression, languages=["en"], page="", elementsbypage=""):
        self.app.REQUEST.set('search', search_expression)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        return browserView.query_result()

    def searchConceptsPageList(self, search_expression, languages=["en"], page="", elementsbypage=""):
        self.app.REQUEST.set('search', search_expression)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        return browserView.paging_header()
    
    def saveConcepts(self, conceptsRelations, openedConcepts="x"):
        return self.azaxview().saveConcepts(conceptsRelations, openedConcepts)

    def conceptsRelationsCorrect(self, conceptsRelations, table):
        t = self.thesaurus_utility()
        for (term, relation, relatedTerms) in table:
            cid = t(term)[0]
            self.assertEqual(t[cid][relation], relatedTerms)

    def afterSetUp(self):
        setSite(self.portal)
        self.login()
        self.setRoles(['Manager', 'Member'])
        self.portal.portal_languages.addSupportedLanguage("es")
        self.loadThesaurus("../pyThesaurus/pyThesaurus/tests/data/open_thesaurus_es.txt", language="es", format="Ding")
    
    def azaxview(self):
        return AzaxView(self.portal, self.app.REQUEST)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestThesaurusAdmin))
    return suite



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: preferences.py 306 2008-06-24 19:40:34Z flarumbe $
#
# end: Platecom header
from zope.interface import implements
from zope.component import getUtility
from OFS.SimpleItem import SimpleItem
from zope.schema.fieldproperty import FieldProperty

import interfaces

class icSemanticManagementThesaurusUpload:
    implements(interfaces.IicSemanticManagementThesaurusUpload)

    def __init__(self, context):
        """
        """
        self.context = context

    def __call__(self):
        pass

    thesaurus_file = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['thesaurus_file'])
    default_language = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['default_language'])
    thesaurus_context = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['thesaurus_context'])
    thesaurus_format = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['thesaurus_format'])
    encoding = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['encoding'])
    new = FieldProperty(interfaces.IicSemanticManagementThesaurusUpload['new'])

class icSemanticVerticalSelectTest:
    implements(interfaces.IicSemanticVerticalSelectTest)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        pass

    vertical_select = FieldProperty(interfaces.IicSemanticVerticalSelectTest['vertical_select'])

def thesaurus_upload_adapter(context):
    return icSemanticManagementThesaurusUpload(context)

def vertical_select_test_adapter(context):
    return icSemanticVerticalSelectTest(context)


# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: test_thesaurus_download.py 261 2008-06-12 21:09:24Z flarumbe $
#
# end: Platecom header
import unittest
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.thesaurus.browser.admin import VerticalSelectTest
from icsemantic.thesaurus.browser.incrementalSearchSelectWidget import IncrementalSearchSelectAzax
from zope.app.component.hooks import setSite

import base

class TestIncrementalSearchSelectWidget(base.icSemanticTestCase):

    def testChooseSelectedConcept(self):
        print self.chooseSelectedConcept("formverticalSelect", "amar@es, antoja")

    def testAutocompleteConcepts(self):
        print self.autocompleteConcepts("formverticalSelect", "amar, temer")

    def testVerticalSelect(self):
        #print self.openVerticalSelectTestPage()
        pass

    def openVerticalSelectTestPage(self):
        vertical_select = VerticalSelectTest(self.portal, self.app.REQUEST)()

    def autocompleteConcepts(self, widgetName, searchExpression):
        return self.incrementalSearchSelectAzax().autocompleteConcepts(widgetName, searchExpression)

    def chooseSelectedConcept(self, widgetName, searchExpression):
        return self.incrementalSearchSelectAzax().chooseSelectedConcept(widgetName, searchExpression)

    def incrementalSearchSelectAzax(self):
        return IncrementalSearchSelectAzax(self.portal, self.app.REQUEST)

    def afterSetUp(self):
        setSite(self.portal)
        self.login()
        self.setRoles(['Manager', 'Member'])
        self.portal.portal_languages.addSupportedLanguage("es")
        self.loadThesaurus("../pyThesaurus/pyThesaurus/tests/data/open_thesaurus_es.txt", language="es", format="Ding")

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestIncrementalSearchSelectWidget))
    return suite



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: base.py 308 2008-06-26 17:46:59Z flarumbe $
#
# end: Platecom header
"""Test setup for integration and functional tests.

When we import PloneTestCase and then call setupPloneSite(), all of Plone's
products are loaded, and a Plone site will be created. This happens at module
level, which makes it faster to run each test, but slows down test runner
startup.
"""
import os, sys
from App import Common

from zope.app.component.hooks import setSite

from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.thesaurus.interfaces import IThesaurus
from icsemantic.catalog.config import *
from pyThesaurus.Concept import Concept
from icsemantic.langfallback.tests import utils

from icsemantic.thesaurus.config import *

ztc.installProduct('GenericSetup')
ztc.installProduct('PloneLanguageTool')
ztc.installProduct('LinguaPlone')

#
# When ZopeTestCase configures Zope, it will *not* auto-load products in
# Products/. Instead, we have to use a statement such as:
#
#   ztc.installProduct('SimpleAttachment')
#
# This does *not* apply to products in eggs and Python packages (i.e. not in
# the Products.*) namespace. For that, see below.
#
# All of Plone's products are already set up by PloneTestCase.
#

@onsetup
def setup_icsemantic_thesaurus():
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', PACKAGE)
    fiveconfigure.debug_mode = False

    # XXX monkey patch everytime (until we figure out the problem where
    #   monkeypatch gets overwritten somewhere)
    try:
        from Products.Five import pythonproducts
        pythonproducts.setupPythonProducts(None)
#        ztc.installProduct(PROJECTNAME)

        # MONKEYPATCH: arregla los problemas con el
        # control panel y la instalacion de Five...
        import App
        App.ApplicationManager.ApplicationManager.Five=utils.Five

        # MONKEYPATCH: arregla los problemas con el
        # HTTP_REFERER en los tests funcionales. Tiene la
        # contra de enviarnos al raiz del plone cada vez
        # que un metodo depende de esa variable, pero es
        # mejor que morir con una excepcion o llenar los
        # tests de try blocks...
        ztc.zopedoctest.functional.http=utils.http


    except ImportError:
        # Not needed in Plone 3
        ztc.installPackage('icsemantic.core')
        ztc.installPackage('icsemantic.langfallback')
        ztc.installPackage(PROJECTNAME)

setup_icsemantic_thesaurus()

ptc.setupPloneSite(products=[PROJECTNAME,])

# TODO: this is copied from icsemantic.catalog. We should move this to
# icsemantic.core.tests.base and use it from there.
def add_test_thesaurus(context):
    """Fill the thesaurus local utility with some useful information"""

    # XXX: this shouldn't be necesary, but doesn't look like something
    # to take care in this package.
    setSite(context)

    t = thesaurus_utility()
    c = Concept(et = ["fútbol@es", "balón pie@es", "soccer@en", "football@en",
                      "football@fr"])
    t.append_concept(c)
    t.append_term("mundial@es", rt=["fútbol@es"], automatic=False)

    t.append_term("pelota@es", rt=["balón pie@es"], contexts=['publicidad'],
                  automatic=False)


class icSemanticTestCase(ptc.PloneTestCase):
    """We use this base class for all the tests in this package. If necessary,
    we can put common utility or setup code in here. This applies to unit
    test cases.
    """

    def loadThesaurus(self, filename, language="en", format='SKOSCore', encoding='latin1'):
        self.thesaurus_utility().clean()
        self.thesaurus_utility().load(filename, language, format=format, encoding=encoding)

    def thesaurus_utility(self):
        return thesaurus_utility()
    
    def _where_are_different(self, aString, anotherString):
        if aString == anotherString:
            print "Both strings are equal."
        else:
            print "String 1:"
            print aString
            print "String 2:"
            print anotherString
            print "length 1: %d" % len(aString)
            print "length 2: %d" % len(anotherString)
            for i in range(min(len(aString), len(anotherString))):
                if aString[i] != anotherString[i]:
                    print "first difference at: %d" % i
                    print "character 1: '%c'" % aString[i]
                    print "character 2: '%c'" % anotherString[i]
                    break

class icSemanticFunctionalTestCase(ptc.FunctionalTestCase):
    """We use this class for functional integration tests that use doctest
    syntax. Again, we can put basic common utility or setup code in here.
    """
    def setUp(self):
        super(ptc.FunctionalTestCase, self).setUp()
        add_test_thesaurus(self.portal)


# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: test_thesaurus_download.py 296 2008-06-18 17:50:17Z flarumbe $
#
# end: Platecom header
import unittest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase.layer import PloneSite

from icsemantic.thesaurus.config import *

from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase.setup import portal_name
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.thesaurus.browser.admin import ThesaurusDownload
from icsemantic.thesaurus.browser.admin import ThesaurusQuery

from zope.app.component.hooks import setSite

import base

class TestThesaurusDownload(base.icSemanticTestCase):


    def warning(self, err):
        import sys
        sys.stderr.write("\nWarning: %s\n" % err)

    def error(self, err):
        import sys
        sys.stderr.write("\nERROR: %s\n" % err)

    def testThesaurusDownloadRDF(self):
        rdf_downloaded = self.thesaurusDownload()
        rdf_expected = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<rdf:RDF xmlns:xml=\"http://www.w3.org/XML/1998/namespace\" xmlns:foaf=\"http://xmlns.com/foaf/0.1/\" xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:skos=\"http://www.w3.org/2004/02/skos/core#\"><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#wetness@en\"><skos:prefLabel xml:lang=\"en\">wetness</skos:prefLabel><skos:altLabel xml:lang=\"en\">dryness</skos:altLabel></skos:Concept><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#shrubs@en\"><skos:prefLabel xml:lang=\"en\">shrubs</skos:prefLabel><skos:altLabel xml:lang=\"en\">bushes</skos:altLabel></skos:Concept><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#rocks@en\"><skos:prefLabel xml:lang=\"en\">rocks</skos:prefLabel><skos:altLabel xml:lang=\"en\">basalt</skos:altLabel><skos:altLabel xml:lang=\"en\">granite</skos:altLabel><skos:altLabel xml:lang=\"en\">slate</skos:altLabel></skos:Concept></rdf:RDF>"
        assert rdf_downloaded == rdf_expected
    
    def testReadConceptsRDF(self):
        rdf_downloaded = self.readTermConcepts("bushes@en")
        rdf_expected = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<rdf:RDF xmlns:xml=\"http://www.w3.org/XML/1998/namespace\" xmlns:foaf=\"http://xmlns.com/foaf/0.1/\" xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:skos=\"http://www.w3.org/2004/02/skos/core#\"><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#shrubs@en\"><skos:prefLabel xml:lang=\"en\">shrubs</skos:prefLabel><skos:altLabel xml:lang=\"en\">bushes</skos:altLabel></skos:Concept></rdf:RDF>"
        assert rdf_downloaded == rdf_expected
    
    def testSearchConceptsRDF(self):
        rdf_downloaded = self.searchConcepts("wet")
        rdf_expected = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<rdf:RDF xmlns:xml=\"http://www.w3.org/XML/1998/namespace\" xmlns:foaf=\"http://xmlns.com/foaf/0.1/\" xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:skos=\"http://www.w3.org/2004/02/skos/core#\"><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#wetness@en\"><skos:prefLabel xml:lang=\"en\">wetness</skos:prefLabel><skos:altLabel xml:lang=\"en\">dryness</skos:altLabel></skos:Concept></rdf:RDF>"
        #self._where_are_different(rdf_downloaded, rdf_expected)
        #return
        assert rdf_downloaded == rdf_expected

        rdf_downloaded = self.searchConcepts("dry")
        rdf_expected = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<rdf:RDF xmlns:xml=\"http://www.w3.org/XML/1998/namespace\" xmlns:foaf=\"http://xmlns.com/foaf/0.1/\" xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:skos=\"http://www.w3.org/2004/02/skos/core#\"><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#wetness@en\"><skos:prefLabel xml:lang=\"en\">wetness</skos:prefLabel><skos:altLabel xml:lang=\"en\">dryness</skos:altLabel></skos:Concept></rdf:RDF>"
        assert rdf_downloaded == rdf_expected

        rdf_downloaded = self.searchConcepts("bas.lt")
        rdf_expected = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<rdf:RDF xmlns:xml=\"http://www.w3.org/XML/1998/namespace\" xmlns:foaf=\"http://xmlns.com/foaf/0.1/\" xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:skos=\"http://www.w3.org/2004/02/skos/core#\"><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#rocks@en\"><skos:prefLabel xml:lang=\"en\">rocks</skos:prefLabel><skos:altLabel xml:lang=\"en\">basalt</skos:altLabel><skos:altLabel xml:lang=\"en\">granite</skos:altLabel><skos:altLabel xml:lang=\"en\">slate</skos:altLabel></skos:Concept></rdf:RDF>"
        assert rdf_downloaded == rdf_expected

        rdf_downloaded = self.searchConcepts("s")
        rdf_expected = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<rdf:RDF xmlns:xml=\"http://www.w3.org/XML/1998/namespace\" xmlns:foaf=\"http://xmlns.com/foaf/0.1/\" xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:dcterms=\"http://purl.org/dc/terms/\" xmlns:skos=\"http://www.w3.org/2004/02/skos/core#\"><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#shrubs@en\"><skos:prefLabel xml:lang=\"en\">shrubs</skos:prefLabel><skos:altLabel xml:lang=\"en\">bushes</skos:altLabel></skos:Concept><skos:Concept rdf:about=\"http://platecom.inter-cultura.com/concepts#rocks@en\"><skos:prefLabel xml:lang=\"en\">rocks</skos:prefLabel><skos:altLabel xml:lang=\"en\">basalt</skos:altLabel><skos:altLabel xml:lang=\"en\">granite</skos:altLabel><skos:altLabel xml:lang=\"en\">slate</skos:altLabel></skos:Concept></rdf:RDF>"
        assert rdf_downloaded == rdf_expected

    def testThesaurusDownloadHTML(self):
        html_downloaded = self.thesaurusConceptsListHTML()
        html_expected = "wetness@en<br>shrubs@en<br>rocks@en<br>"
        assert html_downloaded == html_expected
    
    def testReadConceptsHTML(self):
        html_downloaded = self.readTermConceptsHTML("bushes@en")
        html_expected = "shrubs@en<br>"
        assert html_downloaded == html_expected

    def testSearchConceptsHTML(self):        
        html_downloaded = self.searchConceptsHTML("wet")
        html_expected = "wetness@en<br>"
        assert html_downloaded == html_expected

        html_downloaded = self.searchConceptsHTML("dry")
        html_expected = "wetness@en<br>"
        assert html_downloaded == html_expected

        html_downloaded = self.searchConceptsHTML("bas.lt")
        html_expected = "rocks@en<br>"
        assert html_downloaded == html_expected

        html_downloaded = self.searchConceptsHTML("s")
        html_expected = "shrubs@en<br>rocks@en<br>"
        assert html_downloaded == html_expected
    
    def testReadConceptsHTMLBis(self):
        self.loadThesaurus("../pyThesaurus/pyThesaurus/tests/data/open_thesaurus_es.txt", language="es", format="Ding")

        html_downloaded = self.searchConceptsHTML("#", ["es"])
        html_expected = ""
        assert html_downloaded == html_expected

    def thesaurusDownload(self):
        browserView = ThesaurusDownload(self.portal, self.app.REQUEST)
        return browserView.rdf_string()

    def readTermConcepts(self, term):
        self.app.REQUEST.set('term', term)
        browserView = ThesaurusDownload(self.portal, self.app.REQUEST)
        return browserView.rdf_term_concepts()

    def searchConcepts(self, search_expression):
        self.app.REQUEST.set('search', search_expression)
        browserView = ThesaurusDownload(self.portal, self.app.REQUEST)
        return browserView.rdf_concepts_search()

    def thesaurusConceptsListHTML(self, languages=["en"], page="", elementsbypage=""):
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        return browserView.query_result()

    def readTermConceptsHTML(self, term, languages=["en"], page="", elementsbypage=""):
        self.app.REQUEST.set('term', term)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        return browserView.query_result()

    def searchConceptsHTML(self, search_expression, languages=["en"], page="", elementsbypage=""):
        self.app.REQUEST.set('search', search_expression)
        self.app.REQUEST.set('languages', ",".join(languages))
        self.app.REQUEST.set('format', 'simple')
        if elementsbypage != '' or page != '':
            self.app.REQUEST.set('page', page)
            self.app.REQUEST.set('elementsbypage', elementsbypage)
        else:
            self.app.REQUEST.set('showall', 'true')
        browserView = ThesaurusQuery(self.portal, self.app.REQUEST)
        return browserView.query_result()

    def afterSetUp(self):
        setSite(self.portal)
        self.login()
        self.setRoles(['Manager', 'Member'])
        self.portal.portal_languages.addSupportedLanguage("es")
        self.loadThesaurus("../pyThesaurus/pyThesaurus/tests/data/sample001.rdf")

    def loadThesaurus(self, filename, language="en", format='SKOSCore', encoding='latin1'):
        self.thesaurus_utility().clean()
        self.thesaurus_utility().load(filename, language, format=format, encoding=encoding)

    def thesaurus_utility(self):
        return thesaurus_utility()
    
    def _where_are_different(self, aString, anotherString):
        if aString == anotherString:
            print "Both strings are equal."
        else:
            print "String 1:"
            print aString
            print "String 2:"
            print anotherString
            print "length 1: %d" % len(aString)
            print "length 2: %d" % len(anotherString)
            for i in range(min(len(aString), len(anotherString))):
                if aString[i] != anotherString[i]:
                    print "first difference at: %d" % i
                    print "character 1: '%c'" % aString[i]
                    print "character 2: '%c'" % anotherString[i]
                    break

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestThesaurusDownload))
    return suite



# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: vocabularies.py 256 2008-06-12 15:41:52Z crocha $
#
# end: Platecom header
"""
"""
from zope.schema import vocabulary
from zope.app.schema.vocabulary import IVocabularyFactory
from zope.interface import implements
from zope.component import getUtility
from zope.component import queryUtility

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ITypesTool
from Products.CMFCore.interfaces import IPropertiesTool

from icsemantic.core.i18n import _
from icsemantic.core.config import HAS_PLONE3

from pyThesaurus import io

if HAS_PLONE3:
    from plone.i18n.locales.interfaces import IContentLanguageAvailability
else:
    from Products.PloneLanguageTool import availablelanguages

class ThesaurusFormatVocabulary(object):
	implements(IVocabularyFactory)

	def __call__(self, context):
		formats = []
		for format in io.formats():
			formats.append(vocabulary.SimpleTerm(
				unicode(format),
				title=unicode(format)
				)
			)
	        return vocabulary.SimpleVocabulary( formats )
ThesaurusFormatVocabularyFactory = ThesaurusFormatVocabulary()

class EncodingsVocabulary(object):
	implements(IVocabularyFactory)

	def __call__(self, context):
		encode_list = {
		'ascii':'ascii:English',
		'big5':'big5:Traditional Chinese',
		'big5hkscs':'big5hkscs:Traditional Chinese',
		'cp037':'cp037:English',
		'cp424':'cp424:Hebrew',
		'cp437':'cp437:English',
		'cp500':'cp500:Western Europe',
		'cp737':'cp737:Greek',
		'cp775':'cp775:Baltic languages',
		'cp850':'cp850:Western Europe',
		'cp852':'cp852:Central and Eastern Europe',
		'cp855':'cp855:Bulgarian, Byelorussian, Macedonian, Russian, Serbian',
		'cp856':'cp856:Hebrew',
		'cp857':'cp857:Turkish',
		'cp860':'cp860:Portuguese',
		'cp861':'cp861:Icelandic',
		'cp862':'cp862:Hebrew',
		'cp863':'cp863:Canadian',
		'cp864':'cp864:Arabic',
		'cp865':'cp865:Danish, Norwegian',
		'cp866':'cp866:Russian',
		'cp869':'cp869:Greek',
		'cp874':'cp874:Thai',
		'cp875':'cp875:Greek',
		'cp932':'cp932:Japanese',
		'cp949':'cp949:Korean',
		'cp950':'cp950:Traditional Chinese',
		'cp1006':'cp1006:Urdu',
		'cp1026':'cp1026:Turkish',
		'cp1140':'cp1140:Western Europe',
		'cp1250':'cp1250:Central and Eastern Europe',
		'cp1251':'cp1251:Bulgarian, Byelorussian, Macedonian, Russian, Serbian',
		'cp1252':'cp1252:Western Europe',
		'cp1253':'cp1253:Greek',
		'cp1254':'cp1254:Turkish',
		'cp1255':'cp1255:Hebrew',
		'cp1256':'cp1256:Arabic',
		'cp1257':'cp1257:Baltic languages',
		'cp1258':'cp1258:Vietnamese',
		'euc_jp':'euc jp:Japanese',
		'euc_jis_2004':'euc jis 2004:Japanese',
		'euc_jisx0213':'euc jisx0213:Japanese',
		'euc_kr':'euc kr:Korean',
		'gb2312':'gb2312:Simplified Chinese',
		'gbk':'gbk:Unified Chinese',
		'gb18030':'gb18030:Unified Chinese',
		'hz':'hz:Simplified Chinese',
		'iso2022_jp':'iso2022 jp:Japanese',
		'iso2022_jp_1':'iso2022 jp 1:Japanese',
		'iso2022_jp_2':'iso2022 jp 2:Japanese, Korean, Simplified Chinese, Western Europe, Greek',
		'iso2022_jp_2004':'iso2022 jp 2004:Japanese',
		'iso2022_jp_3':'iso2022 jp 3:Japanese',
		'iso2022_jp_ext':'iso2022 jp ext:Japanese',
		'iso2022_kr':'iso2022 kr:Korean',
		'latin_1':'latin 1:West Europe',
		'iso8859_2':'iso8859 2:Central and Eastern Europe',
		'iso8859_3':'iso8859 3:Esperanto, Maltese',
		'iso8859_4':'iso8859 4:Baltic languagues',
		'iso8859_5':'iso8859 5:Bulgarian, Byelorussian, Macedonian, Russian, Serbian',
		'iso8859_6':'iso8859 6:Arabic',
		'iso8859_7':'iso8859 7:Greek',
		'iso8859_8':'iso8859 8:Hebrew',
		'iso8859_9':'iso8859 9:Turkish',
		'iso8859_10':'iso8859 10:Nordic languages',
		'iso8859_13':'iso8859 13:Baltic languages',
		'iso8859_14':'iso8859 14:Celtic languages',
		'iso8859_15':'iso8859 15:Western Europe',
		'johab':'johab:Korean',
		'koi8_r':'koi8 r:Russian',
		'koi8_u':'koi8 u:Ukrainian',
		'mac_cyrillic':'mac cyrillic:Bulgarian, Byelorussian, Macedonian, Russian, Serbian',
		'mac_greek':'mac greek:Greek',
		'mac_iceland':'mac iceland:Icelandic',
		'mac_latin2':'mac latin2:Central and Eastern Europe',
		'mac_roman':'mac roman:Western Europe',
		'mac_turkish':'mac turkish:Turkish',
		'ptcp154':'ptcp154:Kazakh',
		'shift_jis':'shift jis:Japanese',
		'shift_jis_2004':'shift jis 2004:Japanese',
		'shift_jisx0213':'shift jisx0213:Japanese',
		'utf_16':'utf-16:all languages',
		'utf_16_be':'utf-16 be:all languages (BMP only)',
		'utf_16_le':'utf-16 le:all languages (BMP only)',
		'utf_7':'utf-7:all languages',
		'utf_8':'utf-8:all languages',
		'utf_8_sig':'utf-8 sig:all languages',
		}
		encode = []
		for value, title in encode_list.items():
			encode.append(vocabulary.SimpleTerm(
				unicode(value),
				title=unicode(title)
				)
			)
	        return vocabulary.SimpleVocabulary( encode )
EncodingsVocabularyFactory = EncodingsVocabulary()




# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: test_thesaurus_utility.py 296 2008-06-18 17:50:17Z flarumbe $
#
# end: Platecom header
import unittest

from zope.testing import doctestunit
from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.Five import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase.layer import PloneSite

from icsemantic.thesaurus.config import *
from icsemantic.thesaurus.Thesaurus import thesaurus_utility

from Products.CMFCore.utils import getToolByName
from icsemantic.thesaurus.interfaces import IThesaurus
from icsemantic.thesaurus.Thesaurus import Thesaurus
from Products.PloneTestCase.setup import portal_name
from pyThesaurus.Concept import Concept
from zope.app.component.hooks import setSite

import base

class TestThesaurus(base.icSemanticTestCase):

    def warning(self, err):
        import sys
        sys.stderr.write("\nWarning: %s\n" % err)

    def error(self, err):
        import sys
        sys.stderr.write("\nERROR: %s\n" % err)

    def assertEqualSets(self, a, b):
        a.sort()
        b.sort()
        self.assertEqual(a, b)

    def afterSetUp(self):
        setSite(self.portal)
        if self.thesaurusIsInstalled():
            self.uninstallThesaurus()
        self.removeThesaurusUtility()

    def testInstallThesaurus(self):
        """
        After Thesaurus is installed, Thesaurus Utility should exist.
        """
        assert(not self.existsThesaurusUtility())
        self.installThesaurus()
        assert(self.thesaurusIsInstalled())
        assert(self.existsThesaurusUtility())

    def testUninstallThesaurus(self):
        """
        Thesaurus is uninstalled correctly.
        """
        self.installThesaurus()
        self.uninstallThesaurus()
        assert(not self.thesaurusIsInstalled())

    def testLoadThesaurus(self):
        """
        Thesaurus is loaded at the utility.
        """
        self.installThesaurus()
        ut = self.thesaurusUtility()
        ut.load("../pyThesaurus/pyThesaurus/tests/data/sample000.rdf", "en")
        self.assertPreferedTermsSample000(ut)

    def testLoadThesaurusAndUninstallThesaurus(self):
        """
        Thesaurus is preserved after Thesaurus uninstallation.
        """
        self.installThesaurus()
        ut = self.thesaurusUtility()
        ut.load("../pyThesaurus/pyThesaurus/tests/data/sample000.rdf", "en")
        self.uninstallThesaurus()
        self.assertPreferedTermsSample000(ut)

    def testAppendConcepts(self):
        self.installThesaurus()
        t = self.thesaurusUtility()
        c = Concept(et = ["fútbol@es", "balón pie@es", "soccer@en", "football@en", "football@fr"], rt=["mundial@es"])
        t.append_concept(c)
        t.append_term("mundial@es", rt=["fútbol@es"])
        
        equiv = t.get_equivalent('soccer@en', ['en'], exclude=True)
        self.assertEqual(equiv, ["football@en"])
        
        relat = t.get_related('soccer@en', ['en', 'es'])
        self.assertEqual(relat, ["mundial@es"])
        
        relat = t.get_related('fútbol@es', ['en', 'es'])
        self.assertEqual(relat, ["mundial@es"])

    def testUtilityWrapper(self):
        """Test that the utility wrapper is working"""
        self.installThesaurus()
        try:
            t = thesaurus_utility()
        except NameError:
            self.fail('Thesaurus utility does not exist after install.')
        terms = getattr(t, 'terms', None)
        self.failIf(terms is None, 'Thesaurus utility wrapper not working.')

    def assertPreferedTermsSample000(self, ut):
        """
        Utility has sample000 prefered terms.
        """
        self.assertEqual(ut.get_prefered('animals@en',  ['en']), ['animals@en'])
        self.assertEqual(ut.get_prefered('creatures@en',  ['en']), ['animals@en'])
        self.assertEqual(ut.get_prefered('fauna@en',  ['en']), ['animals@en'])
        self.assertEqual(ut.get_prefered('fauna@en',  ['fr']), [])

    def installThesaurus(self):
        qi = self.portal.portal_quickinstaller
        qi.installProduct("icsemantic.thesaurus")

    def uninstallThesaurus(self):
        qi = self.portal.portal_quickinstaller
        qi.uninstallProducts(("icsemantic.thesaurus",))

    def thesaurusIsInstalled(self):
        qi = self.portal.portal_quickinstaller
        return qi.isProductInstalled("icsemantic.thesaurus")

    def existsThesaurusUtility(self):
        sm = self.portal.getSiteManager()
        return "utilities" in sm.context.objectIds() and "IThesaurus" in sm.context.utilities.objectIds()

    def thesaurusUtility(self):
        return thesaurus_utility()

    def removeThesaurusUtility(self):
        if self.existsThesaurusUtility():
            ut = self.thesaurusUtility()
            fo = ut.getParentNode()
            fo.manage_delObjects("IThesaurus")

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestThesaurus))
    return suite



from zope.app.form.browser.widget import SimpleInputWidget
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.utils import safe_unicode
from icsemantic.thesaurus.Thesaurus import thesaurus_utility
from icsemantic.core.vocabularies import LanguagesVocabularyFactory
import cgi

try:
    from azax import AzaxBaseView, force_unicode
    from azax.plugins.core.interfaces import IKSSCoreCommands
    from azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
except ImportError:
    try:
        from Products.azax import AzaxBaseView, force_unicode
        from Products.azax.plugins.core.interfaces import IKSSCoreCommands
        from Products.azax.plugins.effects.interfaces import IScriptaculousEffectsCommands
    except ImportError:
	from kss.core import force_unicode
        from kss.core.kssview import AzaxBaseView
        from kss.core.plugins.core.interfaces import IKSSCoreCommands
	from kss.core.plugins.effects.interfaces import IScriptaculousEffectsCommands


class IncrementalSearchSelectWidget(SimpleInputWidget):

    template = ViewPageTemplateFile('templates/incrementalSearchSelect-widget.pt')

    def __init__(self, context, request, thesaurus_context="", thesaurus_languages=[]):
        super(IncrementalSearchSelectWidget, self).__init__(context, request)
        self.extra = " autocomplete='off' concepts='' onfocus='timerIncrementalSearchSelect(this.id);' onkeypress='return keypressed(this.id, event)' size='50'"
        self._thesaurus_context = thesaurus_context
        self._thesaurus_languages = thesaurus_languages

    def __call__(self):
        html = self.template()
        html += super(IncrementalSearchSelectWidget, self).__call__()
        html += '<input type="hidden" name="%s_autocompletedExpression" id="%s_autocompletedExpression" value="" concepts="">' % (safe_name(self.name), safe_name(self.name))
        html += '<input type="hidden" name="%s_concepts" id="%s_concepts" value="">' % (safe_name(self.name), safe_name(self.name))
        html += '<input type="hidden" name="%s_autocompleteConceptsAction" id="%s_autocompleteConceptsAction" class="incrementalSearchSelect" widgetName="%s" value="" concepts="">' % (self.name, self.name, self.name)
        html += '<input type="hidden" name="%s_chooseSelectedConceptAction" id="%s_chooseSelectedConceptAction" class="chooseSelectedConcept" widgetName="%s" value="" concepts="">' % (self.name, self.name, self.name)
        html += '<div id="div_%s"></div>' % safe_name(self.name) #TODO: replace to query
        return html
    
    def getInputValue(self):
        return self.selectedConcepts()
    
    def selectedConcepts(self):
        return [ int(strcid) for strcid in self.request["%s_concepts" % safe_name(self.name)].split(",") ]

class IncrementalSearchSelectAzax(AzaxBaseView):

    def autocompleteConcepts(self, widgetName, searchExpression):
        IKSSCoreCommands(self).replaceInnerHTML('div#div_%s' % safe_name(widgetName), self.html_someConceptsList(searchExpression.split(",")))
        return self.render()

    def chooseSelectedConcept(self, widgetName, searchExpression, concepts):
        #TODO: probar .setAttribute
        (newSearchExpression, selectedConcept) = self.html_conceptSelectedSearchExpression(searchExpression)
        IKSSCoreCommands(self).setAttribute('input#%s_autocompletedExpression' % safe_name(widgetName), "value", newSearchExpression)
        if selectedConcept != None:
            if concepts != "":
                concepts += "," + str(selectedConcept)
            else:
                concepts = str(selectedConcept)
            IKSSCoreCommands(self).setAttribute('input#%s_autocompletedExpression' % safe_name(widgetName), "concepts", concepts)
        return self.render()

    def languages(self):
        return [ language.value for language in LanguagesVocabularyFactory(self.context) ]

    def html_conceptSelectedSearchExpression(self, searchExpression):
        search_words = searchExpression.split(",")
        last_word = search_words[len(search_words) - 1].lstrip()
        if last_word == "":
            newSearchExpression = searchExpression
            selectedConcept = None
        else:
            results = self.query(last_word)
            if len(results) > 0:
                selected_word = results[0][0]
                selectedConcept = results[0][1]
                search_words[len(search_words) - 1] = selected_word
                newSearchExpression = (u", ").join(self.lstrip(search_words)) + (u", ")
            else:
                newSearchExpression = searchExpression
                selectedConcept = None
        return (safe_unicode(newSearchExpression), selectedConcept)
    
    def lstrip(self, words):
        return [ word.lstrip() for word in words ]

    def html_someConceptsList(self, search_words):
        last_word = search_words[len(search_words) - 1].lstrip()
        html = ""
        if last_word != "":
            results = self.query(last_word)
            if len(results) > 0:
                selected_word = results[0][0]
            else:
                selected_word = ""
            current_input = last_word

            html += "<table cellspacing='0' cellpadding='0' border='0'>"
            html += "    <tr>"
            html += "        <td><div id='searchResults' class='incrementalSearchSelectResults'>"
            for (term, cid) in results:
                html += self.html_word_item(term, selected_word, current_input)
            if len(results) > self.max_results():
                html += "            ...<br>"
            html += "            </div></td>"
            html += "        <td height='100%'>"
            if len(results) > self.max_results():
                html += "<table cellspacing='0' cellpadding='0' border='0' height='100%'>"
                html += "                <tr><td style='vertical-align: top'><img src='arrowUp.gif' border='0'></td></tr>"
                html += "                <tr><td style='vertical-align: bottom'><img src='arrowDown.gif' border='0'></td></tr>"
                html += "            </table>"
            else:
                html += "&nbsp;"
            html += "</td>"
            html += "    </tr>"
            html += "</table>"
        return html

    def html_word_item(self, word, selected_word, current_input):
        if word == selected_word:
            return u"			<i>%s</i><br>" % self.html_emphasize_input(word, current_input)
        else:
            return u"			%s<br>" % self.html_emphasize_input(word, current_input)

    def html_emphasize_input(self, word, current_input):
        if current_input != "":
            return safe_unicode((u"<b>" + current_input + u"</b>").join(word.split(current_input)))
        else:
            return safe_unicode(word)
    
    def query(self, search_expression):
        return thesaurus_utility().query(search_expression=search_expression,
                                            narrowedthan = None,
                                            borrowedthan = None,
                                            contexts = [],
                                            languages = self.languages(),
                                            inbranch = None,
                                            hidden = None,
                                            max_results = self.max_results())
    
    def max_results(self):
        return 5

def IncrementalSearchSelectWidgetFactory(context, request, thesaurus_context="", thesaurus_languages=[]):
    return IncrementalSearchSelectWidget(context, request, thesaurus_context, thesaurus_languages)

def safe_name(aString):
    return aString.replace(".", "")


# begin: Platecom header
# -*- coding: latin-1 -*-
# vim: set ts=4 sts=4 sw=4 :
#
# $Id: __init__.py 313 2008-06-26 21:09:08Z flarumbe $
#
# end: Platecom header

from Products.CMFCore.DirectoryView import registerDirectory

GLOBALS = globals()
registerDirectory('skins', GLOBALS)

def initialize(context):
    """Initializer called when used as a Zope 2 product."""

    # Apply monkey patches
    import patches
	